@@ -1,7 +1,8 @@
 import os
+from html import escape
 from copy import deepcopy
-from enum import Enum
 from pystac.errors import STACTypeError
+from pystac.html.jinja_env import get_jinja_env
 from typing import (
     Any,
     Callable,
@@ -29,14 +30,20 @@ from pystac.serialization import (
     identify_stac_object,
     migrate_to_latest,
 )
-from pystac.utils import is_absolute_href, make_absolute_href, make_relative_href
+from pystac.utils import (
+    StringEnum,
+    is_absolute_href,
+    make_absolute_href,
+    make_relative_href,
+)
 
 if TYPE_CHECKING:
-    from pystac.item import Asset as Asset_Type, Item as Item_Type
+    from pystac.asset import Asset as Asset_Type
+    from pystac.item import Item as Item_Type
     from pystac.collection import Collection as Collection_Type
 
 
-class CatalogType(str, Enum):
+class CatalogType(StringEnum):
     SELF_CONTAINED = "SELF_CONTAINED"
     """A 'self-contained catalog' is one that is designed for portability.
     Users may want to download an online catalog from and be able to use it on their
@@ -44,7 +51,7 @@ class CatalogType(str, Enum):
 
     See:
         :stac-spec:`The best practices documentation on self-contained catalogs
-            <best-practices.md#self-contained-catalogs>`
+        <best-practices.md#self-contained-catalogs>`
     """
 
     ABSOLUTE_PUBLISHED = "ABSOLUTE_PUBLISHED"
@@ -54,7 +61,7 @@ class CatalogType(str, Enum):
 
     See:
         :stac-spec:`The best practices documentation on published catalogs
-            <best-practices.md#published-catalogs>`
+        <best-practices.md#published-catalogs>`
     """
 
     RELATIVE_PUBLISHED = "RELATIVE_PUBLISHED"
@@ -64,7 +71,7 @@ class CatalogType(str, Enum):
 
     See:
         :stac-spec:`The best practices documentation on published catalogs
-            <best-practices.md#published-catalogs>`
+        <best-practices.md#published-catalogs>`
     """
 
     @classmethod
@@ -110,28 +117,40 @@ class Catalog(STACObject):
     Args:
         id : Identifier for the catalog. Must be unique within the STAC.
         description : Detailed multi-line description to fully explain the catalog.
-            `CommonMark 0.28 syntax <https://commonmark.org/>`_ MAY be used for rich
+            `CommonMark 0.29 syntax <https://commonmark.org/>`_ MAY be used for rich
             text representation.
         title : Optional short descriptive one-line title for the catalog.
         stac_extensions : Optional list of extensions the Catalog implements.
         href : Optional HREF for this catalog, which be set as the
             catalog's self link's HREF.
         catalog_type : Optional catalog type for this catalog. Must
-            be one of the values in :class`~pystac.CatalogType`.
-
-    Attributes:
-        id : Identifier for the catalog.
-        description : Detailed multi-line description to fully explain the catalog.
-        title : Optional short descriptive one-line title for the catalog.
-        stac_extensions : Optional list of extensions the Catalog
-            implements.
-        extra_fields : Extra fields that are part of the top-level JSON
-            properties of the Catalog.
-        links : A list of :class:`~pystac.Link` objects representing
-            all links associated with this Catalog.
-        catalog_type : The catalog type. Defaults to ABSOLUTE_PUBLISHED
+            be one of the values in :class:`~pystac.CatalogType`.
     """
 
+    catalog_type: CatalogType
+    """The catalog type. Defaults to :attr:`CatalogType.ABSOLUTE_PUBLISHED`."""
+
+    description: str
+    """Detailed multi-line description to fully explain the catalog."""
+
+    extra_fields: Dict[str, Any]
+    """Extra fields that are part of the top-level JSON properties of the Catalog."""
+
+    id: str
+    """Identifier for the catalog."""
+
+    links: List[Link]
+    """A list of :class:`~pystac.Link` objects representing all links associated with
+    this Catalog."""
+
+    title: Optional[str]
+    """Optional short descriptive one-line title for the catalog."""
+
+    stac_extensions: List[str]
+    """List of extensions the Catalog implements."""
+
+    _resolved_objects: ResolvedObjectCache
+
     STAC_OBJECT_TYPE = pystac.STACObjectType.CATALOG
 
     _stac_io: Optional[pystac.StacIO] = None
@@ -179,6 +198,14 @@ class Catalog(STACObject):
     def __repr__(self) -> str:
         return "<Catalog id={}>".format(self.id)
 
+    def _repr_html_(self) -> str:
+        jinja_env = get_jinja_env()
+        if jinja_env:
+            template = jinja_env.get_template("Catalog.jinja2")
+            return str(template.render(catalog=self))
+        else:
+            return escape(repr(self))
+
     def set_root(self, root: Optional["Catalog"]) -> None:
         STACObject.set_root(self, root)
         if root is not None:
@@ -186,6 +213,13 @@ class Catalog(STACObject):
                 root._resolved_objects, self._resolved_objects
             )
 
+            # Walk through resolved object links and update the root
+            for link in self.links:
+                if link.rel == pystac.RelType.CHILD or link.rel == pystac.RelType.ITEM:
+                    target = link.target
+                    if isinstance(target, STACObject):
+                        target.set_root(root)
+
     def is_relative(self) -> bool:
         return self.catalog_type in [
             CatalogType.RELATIVE_PUBLISHED,
@@ -461,7 +495,9 @@ class Catalog(STACObject):
         """
         return self.get_links(pystac.RelType.ITEM)
 
-    def to_dict(self, include_self_link: bool = True) -> Dict[str, Any]:
+    def to_dict(
+        self, include_self_link: bool = True, transform_hrefs: bool = True
+    ) -> Dict[str, Any]:
         links = self.links
         if not include_self_link:
             links = [x for x in links if x.rel != pystac.RelType.SELF]
@@ -471,7 +507,7 @@ class Catalog(STACObject):
             "id": self.id,
             "stac_version": pystac.get_stac_version(),
             "description": self.description,
-            "links": [link.to_dict() for link in links],
+            "links": [link.to_dict(transform_href=transform_hrefs) for link in links],
         }
 
         if self.stac_extensions is not None:
@@ -491,7 +527,7 @@ class Catalog(STACObject):
             id=self.id,
             description=self.description,
             title=self.title,
-            stac_extensions=self.stac_extensions,
+            stac_extensions=self.stac_extensions.copy(),
             extra_fields=deepcopy(self.extra_fields),
             catalog_type=self.catalog_type,
         )
@@ -531,6 +567,7 @@ class Catalog(STACObject):
         root_href: str,
         catalog_type: Optional[CatalogType] = None,
         strategy: Optional[HrefLayoutStrategy] = None,
+        stac_io: Optional[pystac.StacIO] = None,
     ) -> None:
         """Normalizes link HREFs to the given root_href, and saves the catalog.
 
@@ -548,9 +585,12 @@ class Catalog(STACObject):
             strategy : The layout strategy to use in setting the
                 HREFS for this catalog. Defaults to
                 :class:`~pystac.layout.BestPracticesLayoutStrategy`
+            stac_io : Optional instance of :class:`~pystac.StacIO` to use. If not
+                provided, will use the instance set while reading in the catalog,
+                or the default instance if this is not available.
         """
         self.normalize_hrefs(root_href, strategy=strategy)
-        self.save(catalog_type)
+        self.save(catalog_type, stac_io=stac_io)
 
     def normalize_hrefs(
         self, root_href: str, strategy: Optional[HrefLayoutStrategy] = None
@@ -665,13 +705,10 @@ class Catalog(STACObject):
         for link in item_links:
             link.resolve_stac_object(root=self.get_root())
             item = cast(pystac.Item, link.target)
-            item_parts = layout_template.get_template_values(item)
+            subcat_ids = layout_template.substitute(item).split("/")
             id_iter = reversed(parent_ids)
             if all(
-                [
-                    "{}".format(id) == next(id_iter, None)
-                    for id in reversed(list(item_parts.values()))
-                ]
+                ["{}".format(id) == next(id_iter, None) for id in reversed(subcat_ids)]
             ):
                 # Skip items for which the sub-catalog structure already
                 # matches the template. The list of parent IDs can include more
@@ -679,12 +716,11 @@ class Catalog(STACObject):
                 keep_item_links.append(link)
                 continue
             curr_parent = self
-            for k, v in item_parts.items():
-                subcat_id = "{}".format(v)
+            for subcat_id in subcat_ids:
                 subcat = curr_parent.get_child(subcat_id)
                 if subcat is None:
-                    subcat_desc = "Catalog of items from {} with {} of {}".format(
-                        curr_parent.id, k, v
+                    subcat_desc = "Catalog of items from {} with id {}".format(
+                        curr_parent.id, subcat_id
                     )
                     subcat = pystac.Catalog(id=subcat_id, description=subcat_desc)
                     curr_parent.add_child(subcat)
@@ -709,6 +745,7 @@ class Catalog(STACObject):
         self,
         catalog_type: Optional[CatalogType] = None,
         dest_href: Optional[str] = None,
+        stac_io: Optional[pystac.StacIO] = None,
     ) -> None:
         """Save this catalog and all it's children/item to files determined by the object's
         self link HREF or a specified path.
@@ -721,6 +758,9 @@ class Catalog(STACObject):
             dest_href : The location where the catalog is to be saved.
                 If not supplied, the catalog's self link HREF is used to determine
                 the location of the catalog file and children's files.
+            stac_io : Optional instance of :class:`~pystac.StacIO` to use. If not
+                provided, will use the instance set while reading in the catalog,
+                or the default instance if this is not available.
         Note:
             If the catalog type is ``CatalogType.ABSOLUTE_PUBLISHED``,
             all self links will be included, and hierarchical links be absolute URLs.
@@ -748,9 +788,12 @@ class Catalog(STACObject):
                     child_dest_href = make_absolute_href(
                         rel_href, dest_href, start_is_dir=True
                     )
-                    child.save(dest_href=child_dest_href)
+                    child.save(
+                        dest_href=os.path.dirname(child_dest_href),
+                        stac_io=stac_io,
+                    )
                 else:
-                    child.save()
+                    child.save(stac_io=stac_io)
 
         for item_link in self.get_item_links():
             if item_link.is_resolved():
@@ -760,9 +803,15 @@ class Catalog(STACObject):
                     item_dest_href = make_absolute_href(
                         rel_href, dest_href, start_is_dir=True
                     )
-                    item.save_object(include_self_link=True, dest_href=item_dest_href)
+                    item.save_object(
+                        include_self_link=items_include_self_link,
+                        dest_href=item_dest_href,
+                        stac_io=stac_io,
+                    )
                 else:
-                    item.save_object(include_self_link=items_include_self_link)
+                    item.save_object(
+                        include_self_link=items_include_self_link, stac_io=stac_io
+                    )
 
         include_self_link = False
         # include a self link if this is the root catalog
@@ -781,7 +830,9 @@ class Catalog(STACObject):
                 rel_href, dest_href, start_is_dir=True
             )
         self.save_object(
-            include_self_link=include_self_link, dest_href=catalog_dest_href
+            include_self_link=include_self_link,
+            dest_href=catalog_dest_href,
+            stac_io=stac_io,
         )
         if catalog_type is not None:
             self.catalog_type = catalog_type
