@@ -1,9 +1,5 @@
-"""Implements the Point Cloud extension.
-
-https://github.com/stac-extensions/pointcloud
-"""
-from enum import Enum
-from typing import Any, Dict, Generic, List, Optional, TypeVar, cast, Union
+"""Implements the :stac-ext:`Point Cloud Extension <pointcloud>`."""
+from typing import Any, Dict, Iterable, Generic, List, Optional, TypeVar, cast, Union
 
 import pystac
 from pystac.extensions.base import (
@@ -13,7 +9,7 @@ from pystac.extensions.base import (
 )
 from pystac.extensions.hooks import ExtensionHooks
 from pystac.summaries import RangeSummary
-from pystac.utils import map_opt, get_required
+from pystac.utils import StringEnum, map_opt, get_required
 
 T = TypeVar("T", pystac.Item, pystac.Asset)
 
@@ -28,7 +24,7 @@ DENSITY_PROP = PREFIX + "density"
 STATISTICS_PROP = PREFIX + "statistics"
 
 
-class PhenomenologyType(str, Enum):
+class PhenomenologyType(StringEnum):
     """Valid values for the ``pc:type`` field in the :stac-ext:`Pointcloud Item
     Properties <pointcloud#item-properties>`."""
 
@@ -39,7 +35,7 @@ class PhenomenologyType(str, Enum):
     OTHER = "other"
 
 
-class SchemaType(str, Enum):
+class SchemaType(StringEnum):
     """Valid values for the ``type`` field in a :stac-ext:`Schema Object
     <pointcloud#schema-object>`."""
 
@@ -55,6 +51,8 @@ class Schema:
     properties.
     """
 
+    properties: Dict[str, Any]
+
     def __init__(self, properties: Dict[str, Any]) -> None:
         self.properties = properties
 
@@ -134,6 +132,8 @@ class Statistic:
     Use :meth:`Statistic.create` to create a new instance of
     ``Statistic`` from property values."""
 
+    properties: Dict[str, Any]
+
     def __init__(self, properties: Dict[str, Any]) -> None:
         self.properties = properties
 
@@ -476,6 +476,9 @@ class ItemPointcloudExtension(PointcloudExtension[pystac.Item]):
     :meth:`PointcloudExtension.ext` on an :class:`~pystac.Item` to extend it.
     """
 
+    item: pystac.Item
+    properties: Dict[str, Any]
+
     def __init__(self, item: pystac.Item):
         self.item = item
         self.properties = item.properties
@@ -493,6 +496,16 @@ class AssetPointcloudExtension(PointcloudExtension[pystac.Asset]):
     :meth:`PointcloudExtension.ext` on an :class:`~pystac.Asset` to extend it.
     """
 
+    asset_href: str
+    """The ``href`` value of the :class:`~pystac.Asset` being extended."""
+
+    properties: Dict[str, Any]
+    """The :class:`~pystac.Asset` fields, including extension properties."""
+
+    additional_read_properties: Optional[Iterable[Dict[str, Any]]] = None
+    """If present, this will be a list containing 1 dictionary representing the
+    properties of the owning :class:`~pystac.Item`."""
+
     def __init__(self, asset: pystac.Asset):
         self.asset_href = asset.href
         self.properties = asset.extra_fields
