@@ -1,7 +1,7 @@
-from enum import Enum
+from pystac.utils import StringEnum
 
 
-class MediaType(str, Enum):
+class MediaType(StringEnum):
     """A list of common media types that can be used in STAC Asset and Link metadata."""
 
     COG = "image/tiff; application=geotiff; profile=cloud-optimized"
@@ -10,6 +10,7 @@ class MediaType(str, Enum):
     GEOTIFF = "image/tiff; application=geotiff"
     HDF = "application/x-hdf"  # Hierarchical Data Format versions 4 and earlier.
     HDF5 = "application/x-hdf5"  # Hierarchical Data Format version 5
+    HTML = "text/html"
     JPEG = "image/jpeg"
     JPEG2000 = "image/jp2"
     JSON = "application/json"
@@ -17,3 +18,4 @@ class MediaType(str, Enum):
     TEXT = "text/plain"
     TIFF = "image/tiff"
     XML = "application/xml"
+    PDF = "application/pdf"
